# How to run:


## Run fix_btree_on_cyrillic_values task

```
mkdir -p /srv/pgsql/csr_backups/fix_btree_on_cyrillic_values/call_db
chown -R postgres:postgres /srv/pgsql/csr_backups

nohup python36 custom_soft_reindex.py \
    --reindex-scenario=fix_btree_on_cyrillic_values.sql \
    --db-name=call_db \
    --track-actions \
    --backup-dir=/srv/pgsql/csr_backups \
    > /dev/null 2>&1 &

```

<code>--backups-dir</code> must be owned by postgres

## Run task to reduce bloat

```
python36 custom_soft_reindex.py \
    --reindex-scenario=fix_bloat.sql \
    --db-name=call_db
```

## Run test

```
python36 custom_soft_reindex.py \
	--test=unit_fk_tests.sql \
	--test-validate=unit_fk_tests_validate.sql \
	--db-name=csr_debug \
	--disable-file-lock
```

Expected output is <code>Integrity checking successfully complete</code>


## Rebuild all indexes exclude indexes with dependent FKs

```
python36 custom_soft_reindex.py \
    --reindex-scenario=unit_fk_tests.sql \
    --skip-fk \
    --db-name=call_db
```


# How to check processed objects:

```
SELECT
	dt,
	c.relname,
	ci.relname
FROM csr_actions a
JOIN pg_namespace n on a.relnamespace = n.oid
JOIN pg_class c on a.indrelid = c.oid
JOIN pg_index i on i.indexrelid = a.indexrelid
JOIN pg_class ci on a.indexrelid = ci.oid
WHERE c.relname = 'yadirect_dic_phrase_value'
	and n.nspname = 'ct_stat_2746_n'
```

# How to debug locks:

1. Set <code>log_level = Debug</code> in <code>conf/custom_soft_reindex.conf</code>
2. Run:

```
SELECT
	-- *_w - is waiting
	-- *_b - is blocker
	s_w.query as waiting_query, 
	s_w.pid as waiting_pid,
	l_w.virtualxid as waiting_virtualxid,
	l_w.virtualtransaction as waiting_virtualtransaction,
	l_w.transactionid as  waiting_transactionid,
	s_w.wait_event as waiting_wait_event,
	---
	s_b.query as blocker_query,
	s_b.pid as blocker_pid,
	l_b.virtualxid as blocker_virtualxid,
	l_b.virtualtransaction as blocker_virtualtransaction,
	l_b.transactionid as  blocker_transactionid,
	s_b.wait_event as blocker_wait_event,
	---
	n.nspname, c.relname, c.relkind, l_b_rel.mode
	
	-- virtualxid		 	Virtual ID of the transaction targeted by the lock, or null if the target is not a virtual transaction ID
	-- virtualtransaction	Virtual ID of the transaction that is holding or awaiting this lock
	-- granted		 		True if lock is held, false if lock is awaited

FROM pg_locks l_w
JOIN pg_stat_activity s_w ON s_w.pid = l_w.pid
JOIN pg_locks l_b ON 
	 l_w.transactionid = l_b.transactionid or l_w.virtualtransaction = l_b.virtualxid
JOIN pg_stat_activity s_b ON s_b.pid = l_b.pid
LEFT JOIN pg_locks l_b_rel ON l_b_rel.virtualtransaction = l_b.virtualxid and l_b_rel.locktype <> 'virtualxid'
LEFT join pg_class c on c.oid = l_b_rel.relation
LEFT join pg_database d on d.oid = l_b_rel.database
LEFT JOIN pg_namespace n on c.relnamespace = n.oid
WHERE NOT l_w.granted AND l_w.pid <> l_b.pid and d.datname = current_database()
ORDER BY l_w.virtualxid
```

About deadlock https://www.postgresql.org/message-id/CAMa1XUhHjCv8Qkx0WOr1Mpm_R4qxN26EibwCrj0Oor2YBUFUTg@mail.gmail.com
