SELECT
    max(n.nspname) as schema_name,
    max(c.relname) as tbl_name,
    max(ci.relname) as idx_name,
    n.oid as relnamespace,
    i.indrelid,
    i.indexrelid,
    max((SELECT pg_get_indexdef(i.indexrelid))) as idxdef,
    i.indisunique,
    i.indisprimary,
    max((SELECT pg_get_constraintdef(cn.oid))) as condef,
    max(cn.oid) as conoid,
    max(cn.conname),
    max(cn.contype),
    max(cn.conrelid)
from pg_class c
INNER JOIN pg_namespace n on c.relnamespace = n.oid
INNER JOIN pg_attribute a on c.oid = a.attrelid
INNER JOIN pg_index i on i.indrelid = c.oid and a.attnum = any( i.indkey )
INNER JOIN pg_class ci on i.indexrelid = ci.oid
INNER JOIN pg_opclass op on i.indclass[0] = op.oid
INNER JOIN pg_am am ON op.opcmethod = am.oid
LEFT JOIN pg_constraint cn ON cn.connamespace = n.oid and cn.conindid = i.indexrelid
WHERE
	n.nspname ilike 'ct_stat_%'
	AND nspname = 'ct_stat_2647_n'       -- uncomment to debug
	--EXT_COND                          -- exclude indexes with postfix_str, this placeholder
	                                    -- will be replaced before executing query
	AND am.amname = 'btree'
	AND (
		(format_type(atttypid, atttypmod) like '%character varying%'
		 or format_type(atttypid, atttypmod) = 'text')
	)
	AND c.relkind = 'r'
	AND a.atttypid <> 0
	AND (
		cn.oid is null
		OR i.indrelid = cn.conrelid
	)
GROUP BY n.oid, i.indexrelid, i.indrelid, i.indisunique, i.indisprimary
ORDER BY schema_name, tbl_name, idx_name    -- required sorting for integrity validation after reindex