-----------------------------------------------------------------------------------------------
drop table if exists  public.tbl_a cascade;
drop table if exists  public.tbl_b cascade;
drop table if exists  public.tbl_c cascade;

CREATE TABLE public.tbl_a
(
    id bigserial,
	tbl_c_id integer,
	some_fld integer,
    CONSTRAINT tbl_a_uniq UNIQUE (id),
	CONSTRAINT tbl_a_uniq_2 UNIQUE (id, some_fld),
    PRIMARY KEY (id)
);

CREATE TABLE public.tbl_c
(
    id bigserial,
    tbl_a_id integer,
	tbl_b_id integer,
	some_fld integer,
	CONSTRAINT tbl_c_uniq_1 UNIQUE (id),
	CONSTRAINT tbl_c_uniq_2 UNIQUE (id, tbl_b_id),
    CONSTRAINT fk_tbl_a_id FOREIGN KEY (tbl_a_id)
        REFERENCES tbl_a(id) ON DELETE CASCADE
);

CREATE TABLE public.tbl_b
(
    id bigserial,
    tbl_c_id integer,
	tbl_a_id integer,
	some_fld integer,
    CONSTRAINT tbl_b_uniq UNIQUE (id),
    CONSTRAINT fk_tbl_c FOREIGN KEY (tbl_c_id)
        REFERENCES tbl_c(id) ON DELETE CASCADE,
    CONSTRAINT fk_tbl_a FOREIGN KEY (tbl_a_id)
        REFERENCES tbl_a(id) ON DELETE CASCADE	
);

ALTER TABLE public.tbl_a
    ADD CONSTRAINT fk_tbl_c FOREIGN KEY (tbl_c_id)
    REFERENCES public.tbl_c (id);

ALTER TABLE public.tbl_c
    ADD CONSTRAINT fk_tbl_b FOREIGN KEY (tbl_b_id)
    REFERENCES public.tbl_b (id);


CREATE UNIQUE INDEX ON tbl_a (id, some_fld, tbl_c_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_a (tbl_c_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_a (tbl_c_id, some_fld) WITH (fillfactor = 90);
CREATE INDEX ON tbl_a (some_fld);

CREATE UNIQUE INDEX ON tbl_b (id, tbl_a_id, tbl_c_id, some_fld) WITH (fillfactor = 90);
CREATE INDEX ON tbl_b (tbl_a_id, tbl_c_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_b (id, tbl_a_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_b (some_fld);

CREATE UNIQUE INDEX ON tbl_c (id, tbl_b_id, some_fld) WITH (fillfactor = 90);
CREATE INDEX ON tbl_c (tbl_b_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_c (tbl_b_id, some_fld) WITH (fillfactor = 90);
CREATE INDEX ON tbl_c (tbl_b_id) where tbl_b_id > 500;

INSERT INTO tbl_a (some_fld) SELECT 1000 from(SELECT generate_series(1,10000) as some_fld) T;
INSERT INTO tbl_b (some_fld) SELECT 1000 from(SELECT generate_series(1,10000) as some_fld) T;
INSERT INTO tbl_c (some_fld) SELECT 1000 from(SELECT generate_series(1,10000) as some_fld) T;

update tbl_a set tbl_c_id = id;
update tbl_b set tbl_c_id = id, tbl_a_id = id;
update tbl_c set tbl_a_id = id, tbl_b_id = id;

ALTER TABLE public.tbl_c
    ADD CONSTRAINT tbl_c_uniq PRIMARY KEY (id, tbl_b_id);

CREATE INDEX long_name_name_name_name_name_name_name_name_name_name_name_nam ON tbl_a (some_fld);  -- max name
-----------------------------------------------------------------------------------------------

-- list all indexes
select ci.relname, i.indisprimary, i.indisunique, (SELECT pg_get_indexdef(i.indexrelid)) as def
from pg_index i
JOIN pg_class AS ci ON i.indexrelid = ci.oid and ci.relkind = 'i'
JOIN pg_namespace n ON n.oid = ci.relnamespace
WHERE n.nspname NOT IN ('pg_catalog', 'information_schema', 'pg_toast')
order by ci.relname;

/*
"CREATE UNIQUE INDEX tbl_a_id_some_fld_tbl_c_id_idx ON public.tbl_a USING btree (id, some_fld, tbl_c_id) WITH (fillfactor='90')"
"CREATE INDEX tbl_a_some_fld_idx ON public.tbl_a USING btree (some_fld)"
"CREATE INDEX tbl_a_tbl_c_id_idx ON public.tbl_a USING btree (tbl_c_id) WITH (fillfactor='90')"
"CREATE INDEX tbl_a_tbl_c_id_some_fld_idx ON public.tbl_a USING btree (tbl_c_id, some_fld) WITH (fillfactor='90')"
"CREATE UNIQUE INDEX tbl_a_uniq ON public.tbl_a USING btree (id)"
"CREATE UNIQUE INDEX tbl_a_uniq_2 ON public.tbl_a USING btree (id, some_fld)"
"CREATE INDEX tbl_b_id_tbl_a_id_idx ON public.tbl_b USING btree (id, tbl_a_id) WITH (fillfactor='90')"
"CREATE UNIQUE INDEX tbl_b_id_tbl_a_id_tbl_c_id_some_fld_idx ON public.tbl_b USING btree (id, tbl_a_id, tbl_c_id, some_fld) WITH (fillfactor='90')"
"CREATE INDEX tbl_b_some_fld_idx ON public.tbl_b USING btree (some_fld)"
"CREATE INDEX tbl_b_tbl_a_id_tbl_c_id_idx ON public.tbl_b USING btree (tbl_a_id, tbl_c_id) WITH (fillfactor='90')"
"CREATE UNIQUE INDEX tbl_b_uniq ON public.tbl_b USING btree (id)"
"CREATE UNIQUE INDEX tbl_c_id_tbl_b_id_some_fld_idx ON public.tbl_c USING btree (id, tbl_b_id, some_fld) WITH (fillfactor='90')"
"CREATE INDEX tbl_c_tbl_b_id_idx ON public.tbl_c USING btree (tbl_b_id) WITH (fillfactor='90')"
"CREATE INDEX tbl_c_tbl_b_id_idx1 ON public.tbl_c USING btree (tbl_b_id) WHERE (tbl_b_id > 500)"
"CREATE INDEX tbl_c_tbl_b_id_some_fld_idx ON public.tbl_c USING btree (tbl_b_id, some_fld) WITH (fillfactor='90')"
"CREATE UNIQUE INDEX tbl_c_uniq ON public.tbl_c USING btree (id, tbl_b_id)"
"CREATE UNIQUE INDEX tbl_c_uniq_1 ON public.tbl_c USING btree (id)"
"CREATE UNIQUE INDEX tbl_c_uniq_2 ON public.tbl_c USING btree (id, tbl_b_id)"
*/


-- constraints
select ci.relname, i.indisprimary, i.indisunique, array_agg(cn.contype), array_agg((select pg_get_constraintdef(cn.oid))) as constraintdef
from pg_index i
JOIN pg_class AS ci ON i.indexrelid = ci.oid and ci.relkind = 'i'
JOIN pg_namespace n ON n.oid = ci.relnamespace
JOIN pg_constraint cn ON cn.connamespace = n.oid and cn.conindid = i.indexrelid
WHERE n.nspname NOT IN ('pg_catalog', 'information_schema', 'pg_toast')
group by ci.oid, ci.relname, i.indisprimary, i.indisunique
order by ci.relname;

/*
"tbl_a_uniq"	true	true	"{f,f,p}"	"{"FOREIGN KEY (tbl_a_id) REFERENCES tbl_a(id) ON DELETE CASCADE","FOREIGN KEY (tbl_a_id) REFERENCES tbl_a(id) ON DELETE CASCADE","PRIMARY KEY (id)"}"
"tbl_a_uniq_2"	false	true	"{u}"	"{"UNIQUE (id, some_fld)"}"
"tbl_b_uniq"	false	true	"{u,f}"	"{"UNIQUE (id)","FOREIGN KEY (tbl_b_id) REFERENCES tbl_b(id)"}"
"tbl_c_uniq"	true	true	"{p}"	"{"PRIMARY KEY (id, tbl_b_id)"}"
"tbl_c_uniq_1"	false	true	"{f,u,f}"	"{"FOREIGN KEY (tbl_c_id) REFERENCES tbl_c(id) ON DELETE CASCADE","UNIQUE (id)","FOREIGN KEY (tbl_c_id) REFERENCES tbl_c(id)"}"
"tbl_c_uniq_2"	false	true	"{u}"	"{"UNIQUE (id, tbl_b_id)"}"
*/

/*
ALTER TABLE
    ADD constraint_name [ NOT VALID ]
    ADD CONSTRAINT constraint_name
		{ UNIQUE | PRIMARY KEY } USING INDEX index_name
		[ DEFERRABLE | NOT DEFERRABLE ] [ INITIALLY DEFERRED | INITIALLY IMMEDIATE ]

    VALIDATE CONSTRAINT constraint_name
    DROP CONSTRAINT [ IF EXISTS ]  constraint_name [ RESTRICT | CASCADE ]

*/
--  ALTER TABLE "%s" DROP CONSTRAINT "%s",
  -- ADD CONSTRAINT "%s" "%s" USING INDEX "%s"
		-- PRIMARY KEY
		-- UNIQUE

-----------------------------------------------------------------------------------------------

-- create new indexes
CREATE UNIQUE INDEX tbl_a_id_some_fld_tbl_c_id_idx_new ON public.tbl_a USING btree (id, some_fld, tbl_c_id) WITH (fillfactor='90');
CREATE INDEX tbl_a_some_fld_idx_new ON public.tbl_a USING btree (some_fld);
CREATE INDEX tbl_a_tbl_c_id_idx_new ON public.tbl_a USING btree (tbl_c_id) WITH (fillfactor='90');
CREATE INDEX tbl_a_tbl_c_id_some_fld_idx_new ON public.tbl_a USING btree (tbl_c_id, some_fld) WITH (fillfactor='90');
CREATE UNIQUE INDEX tbl_a_uniq_new ON public.tbl_a USING btree (id);
CREATE UNIQUE INDEX tbl_a_uniq_2_new ON public.tbl_a USING btree (id, some_fld);
CREATE INDEX tbl_b_id_tbl_a_id_idx_new ON public.tbl_b USING btree (id, tbl_a_id) WITH (fillfactor='90');
CREATE UNIQUE INDEX tbl_b_id_tbl_a_id_tbl_c_id_some_fld_idx_new ON public.tbl_b USING btree (id, tbl_a_id, tbl_c_id, some_fld) WITH (fillfactor='90');
CREATE INDEX tbl_b_some_fld_idx_new ON public.tbl_b USING btree (some_fld);
CREATE INDEX tbl_b_tbl_a_id_tbl_c_id_idx_new ON public.tbl_b USING btree (tbl_a_id, tbl_c_id) WITH (fillfactor='90');
CREATE UNIQUE INDEX tbl_b_uniq_new ON public.tbl_b USING btree (id);
CREATE UNIQUE INDEX tbl_c_id_tbl_b_id_some_fld_idx_new ON public.tbl_c USING btree (id, tbl_b_id, some_fld) WITH (fillfactor='90');
CREATE INDEX tbl_c_tbl_b_id_idx_new ON public.tbl_c USING btree (tbl_b_id) WITH (fillfactor='90');
CREATE INDEX tbl_c_tbl_b_id_idx1_new ON public.tbl_c USING btree (tbl_b_id) WHERE (tbl_b_id > 500);
CREATE INDEX tbl_c_tbl_b_id_some_fld_idx_new ON public.tbl_c USING btree (tbl_b_id, some_fld) WITH (fillfactor='90');
CREATE UNIQUE INDEX tbl_c_uniq_new ON public.tbl_c USING btree (id, tbl_b_id);
CREATE UNIQUE INDEX tbl_c_uniq_1_new ON public.tbl_c USING btree (id);
CREATE UNIQUE INDEX tbl_c_uniq_2_new ON public.tbl_c USING btree (id, tbl_b_id);

-- drop old
DROP INDEX tbl_a_id_some_fld_tbl_c_id_idx; -- ок
DROP INDEX tbl_a_some_fld_idx; -- ок
DROP INDEX tbl_a_tbl_c_id_idx; -- ок
DROP INDEX tbl_a_tbl_c_id_some_fld_idx; -- ок
-- 1 -- DROP INDEX tbl_a_uniq; -- cannot drop index tbl_a_uniq because constraint tbl_a_uniq on table tbl_a requires it
-- 2 -- DROP INDEX tbl_a_uniq_2; -- cannot drop index tbl_a_uniq_2 because constraint tbl_a_uniq_2 on table tbl_a requires it
DROP INDEX tbl_b_id_tbl_a_id_idx; -- ок
DROP INDEX tbl_b_id_tbl_a_id_tbl_c_id_some_fld_idx; -- ок
DROP INDEX tbl_b_some_fld_idx; -- ок
DROP INDEX tbl_b_tbl_a_id_tbl_c_id_idx; -- ок
-- 3 -- DROP INDEX tbl_b_uniq; -- cannot drop index tbl_b_uniq because constraint tbl_b_uniq on table tbl_b requires it
DROP INDEX tbl_c_id_tbl_b_id_some_fld_idx; -- ок
DROP INDEX tbl_c_tbl_b_id_idx; -- ок
DROP INDEX tbl_c_tbl_b_id_idx1; -- ок
DROP INDEX tbl_c_tbl_b_id_some_fld_idx; -- ок
-- 4 -- DROP INDEX tbl_c_uniq; -- cannot drop index tbl_c_uniq because constraint tbl_c_uniq on table tbl_c requires it
-- 5 -- DROP INDEX tbl_c_uniq_1; --cannot drop index tbl_c_uniq_1 because constraint tbl_c_uniq_1 on table tbl_c requires it
-- 6 -- DROP INDEX tbl_c_uniq_2; -- cannot drop index tbl_c_uniq_2 because constraint tbl_c_uniq_2 on table tbl_c requires it

-----------------------------------------------------------------------------------
-- check constraints by index name
select
	n.nspname,
	cr.relname,
	ci.relname,
	i.indisprimary,
	i.indisunique,
	cn.contype,
	(select pg_get_constraintdef(cn.oid)) as constraintdef,
	nc.nspname,
	cc.relname,
	cn.conname
from pg_index i
JOIN pg_class AS ci ON i.indexrelid = ci.oid and ci.relkind = 'i'
JOIN pg_class AS cr ON i.indrelid = cr.oid and cr.relkind = 'r'
JOIN pg_namespace n ON n.oid = ci.relnamespace
JOIN pg_constraint cn ON cn.connamespace = n.oid and cn.conindid = i.indexrelid
JOIN pg_class AS cc ON cn.conrelid = cc.oid
JOIN pg_namespace AS nc ON nc.oid = cc.relnamespace
--WHERE ci.relname = 'tbl_a_uniq'
ORDER BY cn.contype, cn.conname;
-----------------------------------------------------------------------------------
-- 1 -- tbl_a_uniq
/*
"public"	"tbl_a"	"tbl_a_uniq"	true	true	"f"	"FOREIGN KEY (tbl_a_id) REFERENCES tbl_a(id) ON DELETE CASCADE"	"public"	"tbl_b"	"fk_tbl_a"
"public"	"tbl_a"	"tbl_a_uniq"	true	true	"f"	"FOREIGN KEY (tbl_a_id) REFERENCES tbl_a(id) ON DELETE CASCADE"	"public"	"tbl_c"	"fk_tbl_a_id"
"public"	"tbl_a"	"tbl_a_uniq"	true	true	"p"	"PRIMARY KEY (id)"	"public"	"tbl_a"	"tbl_a_uniq"
*/
---
ALTER TABLE public.tbl_b
    DROP CONSTRAINT fk_tbl_a;

ALTER TABLE public.tbl_c
    DROP CONSTRAINT fk_tbl_a_id;
---
ALTER TABLE public.tbl_a
  DROP CONSTRAINT tbl_a_uniq,
  ADD CONSTRAINT tbl_a_uniq PRIMARY KEY USING INDEX tbl_a_uniq_new;
---
ALTER TABLE public.tbl_b
	ADD CONSTRAINT fk_tbl_a
	FOREIGN KEY (tbl_a_id) REFERENCES tbl_a(id) ON DELETE CASCADE NOT VALID;

ALTER TABLE public.tbl_b
	VALIDATE CONSTRAINT fk_tbl_a;

ALTER TABLE public.tbl_c
	ADD CONSTRAINT fk_tbl_a_id
	FOREIGN KEY (tbl_a_id) REFERENCES tbl_a(id) ON DELETE CASCADE NOT VALID;

ALTER TABLE public.tbl_c
	VALIDATE CONSTRAINT fk_tbl_a_id;
---

--  recheck constraints, validation ok
/*
"public"	"tbl_a"	"tbl_a_uniq"	true	true	"f"	"FOREIGN KEY (tbl_a_id) REFERENCES tbl_a(id) ON DELETE CASCADE"	"public"	"tbl_b"	"fk_tbl_a"
"public"	"tbl_a"	"tbl_a_uniq"	true	true	"f"	"FOREIGN KEY (tbl_a_id) REFERENCES tbl_a(id) ON DELETE CASCADE"	"public"	"tbl_c"	"fk_tbl_a_id"
"public"	"tbl_a"	"tbl_a_uniq"	true	true	"p"	"PRIMARY KEY (id)"	"public"	"tbl_a"	"tbl_a_uniq"
*/
-----------------------------------------------------------------------------------
-- 2 -- tbl_a_uniq_2
-- "public"	"tbl_a"	"tbl_a_uniq_2"	false	true	"u"	"UNIQUE (id, some_fld)"	"public"	"tbl_a"	"tbl_a_uniq_2"

ALTER TABLE public.tbl_a
  DROP CONSTRAINT tbl_a_uniq_2,
  ADD CONSTRAINT tbl_a_uniq_2 UNIQUE USING INDEX tbl_a_uniq_2_new;

-- validation ok
-----------------------------------------------------------------------------------
-- 3 -- tbl_b_uniq
--"public"	"tbl_b"	"tbl_b_uniq"	false	true	"f"	"FOREIGN KEY (tbl_b_id) REFERENCES tbl_b(id)"	"public"	"tbl_c"	"fk_tbl_b"
--"public"	"tbl_b"	"tbl_b_uniq"	false	true	"u"	"UNIQUE (id)"									"public"	"tbl_b"	"tbl_b_uniq"

ALTER TABLE public.tbl_c
    DROP CONSTRAINT fk_tbl_b;

ALTER TABLE public.tbl_b
  DROP CONSTRAINT tbl_b_uniq,
  ADD CONSTRAINT tbl_b_uniq UNIQUE USING INDEX tbl_b_uniq_new;

ALTER TABLE public.tbl_c
	ADD CONSTRAINT fk_tbl_b
	FOREIGN KEY (tbl_b_id) REFERENCES tbl_b(id) NOT VALID;

ALTER TABLE public.tbl_c
	VALIDATE CONSTRAINT fk_tbl_b;

-- validation ok
-----------------------------------------------------------------------------------
-- 4 -- tbl_c_uniq
-- "public"	"tbl_c"	"tbl_c_uniq"	true	true	"p"	"PRIMARY KEY (id, tbl_b_id)"	"public"	"tbl_c"	"tbl_c_uniq"

ALTER TABLE public.tbl_c
  DROP CONSTRAINT tbl_c_uniq,
  ADD CONSTRAINT tbl_c_uniq PRIMARY KEY USING INDEX tbl_c_uniq_new;

-- validation ok
-----------------------------------------------------------------------------------
-- 5 -- tbl_c_uniq_1
/*
"public"	"tbl_c"	"tbl_c_uniq_1"	false	true	"f"	"FOREIGN KEY (tbl_c_id) REFERENCES tbl_c(id)"	"public"	"tbl_a"	"fk_tbl_c"
"public"	"tbl_c"	"tbl_c_uniq_1"	false	true	"f"	"FOREIGN KEY (tbl_c_id) REFERENCES tbl_c(id) ON DELETE CASCADE"	"public"	"tbl_b"	"fk_tbl_c"
"public"	"tbl_c"	"tbl_c_uniq_1"	false	true	"u"	"UNIQUE (id)"	"public"	"tbl_c"	"tbl_c_uniq_1"
*/

ALTER TABLE public.tbl_a
    DROP CONSTRAINT fk_tbl_c;

ALTER TABLE public.tbl_b
    DROP CONSTRAINT fk_tbl_c;
---
ALTER TABLE public.tbl_c
  DROP CONSTRAINT tbl_c_uniq_1,
  ADD CONSTRAINT tbl_c_uniq_1 UNIQUE USING INDEX tbl_c_uniq_1_new;
---
ALTER TABLE public.tbl_a
	ADD CONSTRAINT fk_tbl_c
	FOREIGN KEY (tbl_c_id) REFERENCES tbl_c(id) NOT VALID;

ALTER TABLE public.tbl_a
	VALIDATE CONSTRAINT fk_tbl_c;

ALTER TABLE public.tbl_b
	ADD CONSTRAINT fk_tbl_c
	FOREIGN KEY (tbl_c_id) REFERENCES tbl_c(id) ON DELETE CASCADE NOT VALID;

ALTER TABLE public.tbl_b
	VALIDATE CONSTRAINT fk_tbl_c;

-- validation ok
-----------------------------------------------------------------------------------
-- 6 -- tbl_c_uniq_2
--"public"	"tbl_c"	"tbl_c_uniq_2"	false	true	"u"	"UNIQUE (id, tbl_b_id)"	"public"	"tbl_c"	"tbl_c_uniq_2"

ALTER TABLE public.tbl_c
  DROP CONSTRAINT tbl_c_uniq_2,
  ADD CONSTRAINT tbl_c_uniq_2 UNIQUE USING INDEX tbl_c_uniq_2_new;

-- validation ok
-----------------------------------------------------------------------------------

-- CSR Common Logic

-- get indexes - FldsEnumBasicList
select
	n.nspname,
	cr.relname,
	ci.relname,
	(SELECT pg_get_indexdef (i.indexrelid)) as def,
	i.indexrelid,
	i.indrelid,
	exists(select 1 from pg_constraint cn where cn.connamespace = n.oid and cn.conindid = i.indexrelid) as complex
from pg_index i
JOIN pg_class AS ci ON i.indexrelid = ci.oid and ci.relkind = 'i'
JOIN pg_class AS cr ON i.indrelid = cr.oid and cr.relkind in ('r', 'm')
JOIN pg_namespace n ON n.oid = ci.relnamespace
WHERE n.nspname not in ('pg_catalog', 'pg_toast')
ORDER BY n.nspname, cr.relname, ci.relname;

-- for each complex - FldsEnumComplex
select
	n.nspname,
	cr.relname,
	ci.relname,
	i.indisprimary,
	i.indisunique,
	cn.contype,
	(select pg_get_constraintdef(cn.oid)) as constraintdef,
	nc.nspname,
	cc.relname,
	cn.conname
from pg_index i
JOIN pg_class AS ci ON i.indexrelid = ci.oid and ci.relkind = 'i'
JOIN pg_class AS cr ON i.indrelid = cr.oid and cr.relkind in ('r', 'm')
JOIN pg_namespace n ON n.oid = ci.relnamespace
JOIN pg_constraint cn ON cn.connamespace = n.oid and cn.conindid = i.indexrelid
JOIN pg_class AS cc ON cn.conrelid = cc.oid
JOIN pg_namespace AS nc ON nc.oid = cc.relnamespace
WHERE n.nspname = 'public' and cr.relname = 'tbl_a' and ci.relname = 'tbl_a_uniq'
ORDER BY cn.contype, cn.conname;

-- for each item of complex case

-- if contype = f: drop all FKs in one tx
ALTER TABLE rec[7].rec[8]
    DROP CONSTRAINT rec[9];

-- if contype in (p, u)
ALTER TABLE rec[7].rec[8]
  DROP CONSTRAINT rec[9],
  ADD CONSTRAINT rec[9] rec[6] USING INDEX rec[2]_new;

-- if contype = f: recreate all FKs two transactions per each FK
ALTER TABLE rec[7].rec[8]
	ADD CONSTRAINT rec[9]
	rec[6] NOT VALID; -- tx 1

ALTER TABLE rec[7].rec[8]
	VALIDATE CONSTRAINT rec[9]; -- tx 2

-- validation query - FldsEnumIntegrity
select
	n.nspname,
	cr.relname,
	ci.relname,
	i.indisprimary,
	i.indisunique,
	(SELECT pg_get_indexdef (i.indexrelid)) as def,
	cn.contype,
	(select pg_get_constraintdef(cn.oid)) as constraintdef,
	nc.nspname,
	cc.relname,
	cn.conname
from pg_index i
JOIN pg_class AS ci ON i.indexrelid = ci.oid and ci.relkind = 'i'
JOIN pg_class AS cr ON i.indrelid = cr.oid and cr.relkind in ('r', 'm')
JOIN pg_namespace n ON n.oid = ci.relnamespace
LEFT JOIN pg_constraint cn ON cn.connamespace = n.oid and cn.conindid = i.indexrelid
LEFT JOIN pg_class AS cc ON cn.conrelid = cc.oid
LEFT JOIN pg_namespace AS nc ON nc.oid = cc.relnamespace
WHERE n.nspname not in ('pg_catalog', 'pg_toast')
ORDER BY n.nspname, cr.relname, ci.relname;
