-----------------------------------------------------------------------------------------------
CREATE SCHEMA IF NOT EXISTS ext;

drop table if exists  public.tbl_a cascade;
drop table if exists  public.tbl_b cascade;
drop table if exists  public.tbl_c cascade;
drop table if exists  ext.tbl_c_ext cascade;

CREATE TABLE public.tbl_a
(
    id bigserial,
    tbl_c_id integer,
    some_fld integer,
    CONSTRAINT tbl_a_uniq UNIQUE (id),
    CONSTRAINT tbl_a_uniq_2 UNIQUE (id, some_fld),
    PRIMARY KEY (id)
);

CREATE TABLE public.tbl_c
(
    id bigserial,
    tbl_a_id integer,
    tbl_b_id integer,
    some_fld integer,
    CONSTRAINT tbl_c_uniq_1 UNIQUE (id),
    CONSTRAINT tbl_c_uniq_2 UNIQUE (id, tbl_b_id),
    CONSTRAINT fk_tbl_a_id FOREIGN KEY (tbl_a_id)
        REFERENCES tbl_a(id) ON DELETE CASCADE
);

CREATE TABLE public.tbl_b
(
    id bigserial,
    tbl_c_id integer,
    tbl_a_id integer,
    some_fld integer,
    CONSTRAINT tbl_b_uniq UNIQUE (id),
    CONSTRAINT fk_tbl_c FOREIGN KEY (tbl_c_id)
        REFERENCES tbl_c(id) ON DELETE CASCADE,
    CONSTRAINT fk_tbl_a FOREIGN KEY (tbl_a_id)
        REFERENCES tbl_a(id) ON DELETE CASCADE
);

CREATE TABLE ext.tbl_c_ext
(
    id bigserial,
    tbl_a_id integer,
    tbl_b_id integer,
    some_fld integer,
    CONSTRAINT tbl_c_ext_uniq_1 UNIQUE (id),
    CONSTRAINT tbl_c_ext_uniq_2 UNIQUE (id, tbl_b_id),
    CONSTRAINT fk_tbl_a_id FOREIGN KEY (tbl_a_id)
        REFERENCES public.tbl_a(id) ON DELETE CASCADE
);

ALTER TABLE public.tbl_a
    ADD CONSTRAINT fk_tbl_c FOREIGN KEY (tbl_c_id)
    REFERENCES public.tbl_c (id);

ALTER TABLE public.tbl_c
    ADD CONSTRAINT fk_tbl_b FOREIGN KEY (tbl_b_id)
    REFERENCES public.tbl_b (id);

ALTER TABLE public.tbl_a
    ADD CONSTRAINT fk_tbl_c_ext FOREIGN KEY (tbl_c_id)
    REFERENCES ext.tbl_c_ext (id);

ALTER TABLE ext.tbl_c_ext
    ADD CONSTRAINT fk_tbl_b_ext FOREIGN KEY (tbl_b_id)
    REFERENCES public.tbl_b (id);


CREATE UNIQUE INDEX ON tbl_a (id, some_fld, tbl_c_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_a (tbl_c_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_a (tbl_c_id, some_fld) WITH (fillfactor = 90);
CREATE INDEX ON tbl_a (some_fld);

CREATE UNIQUE INDEX ON tbl_b (id, tbl_a_id, tbl_c_id, some_fld) WITH (fillfactor = 90);
CREATE INDEX ON tbl_b (tbl_a_id, tbl_c_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_b (id, tbl_a_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_b (some_fld);

CREATE UNIQUE INDEX ON tbl_c (id, tbl_b_id, some_fld) WITH (fillfactor = 90);
CREATE INDEX ON tbl_c (tbl_b_id) WITH (fillfactor = 90);
CREATE INDEX ON tbl_c (tbl_b_id, some_fld) WITH (fillfactor = 90);
CREATE INDEX ON tbl_c (tbl_b_id) where tbl_b_id > 500;

CREATE UNIQUE INDEX ON ext.tbl_c_ext (id, tbl_b_id, some_fld) WITH (fillfactor = 90);

INSERT INTO tbl_a (some_fld) SELECT 1000 from(SELECT generate_series(1,100000) as some_fld) T;
INSERT INTO tbl_b (some_fld) SELECT 1000 from(SELECT generate_series(1,100000) as some_fld) T;
INSERT INTO tbl_c (some_fld) SELECT 1000 from(SELECT generate_series(1,100000) as some_fld) T;
INSERT INTO ext.tbl_c_ext (some_fld) SELECT 1000 from(SELECT generate_series(1,100000) as some_fld) T;

update tbl_a set tbl_c_id = id;
update tbl_b set tbl_c_id = id, tbl_a_id = id;
update tbl_c set tbl_a_id = id, tbl_b_id = id;
update ext.tbl_c_ext set tbl_a_id = id, tbl_b_id = id;

ALTER TABLE public.tbl_c
    ADD CONSTRAINT tbl_c_uniq PRIMARY KEY (id, tbl_b_id);

ALTER TABLE ext.tbl_c_ext
    ADD CONSTRAINT tbl_c_uniq PRIMARY KEY (id, tbl_b_id);

CREATE INDEX long_name_name_name_name_name_name_name_name_name_name_name_nam ON tbl_a (some_fld);  -- max name
-----------------------------------------------------------------------------------------------

do $$
begin
   IF NOT EXISTS (SELECT FROM pg_catalog.pg_roles WHERE rolname = 'csr_test') THEN
      CREATE USER csr_test LOGIN PASSWORD 'csr_test';
   END IF;
end$$;

CREATE OR REPLACE VIEW tbl_a_v AS SELECT a.* from tbl_a a group by a.id;
CREATE OR REPLACE VIEW tbl_b_v AS SELECT a.* from tbl_a a join tbl_b b on a.id = b.id group by a.id;
CREATE OR REPLACE VIEW tbl_c_v AS SELECT a.* from tbl_a a join tbl_c c on a.id = c.id group by a.id;
CREATE OR REPLACE VIEW tbl_ab_v AS SELECT a.* from tbl_a a join tbl_b b on a.id = b.id group by a.id;

GRANT UPDATE, SELECT ON TABLE tbl_a_v TO csr_test;
GRANT DELETE, UPDATE, SELECT, INSERT ON TABLE tbl_b_v TO csr_test;
GRANT ALL ON TABLE tbl_c_v TO csr_test;
GRANT UPDATE, SELECT ON TABLE tbl_ab_v TO csr_test;
